'''
Created on 16.1.2011

@author: myron
'''

import string
#from string import split
from ModelPart import ModelPart
from numpy import *
from Quaternion import Quaternion
from OpenGL.GL import *
import time

class Model(object):
    '''
    Model of spheres with physical properties
    
    postion
    orientation
    mass
    center of mass
    
    linear velocity
    angular velocity
    
    linear acceleration
    angular acceleration
    
    torque
    '''
    
    parts = [] # list of model parts
    
    forceEndTime = -1

    def __init__(self, filename):
        '''
        Constructor
        '''
        self.parts = []
        self.lastUpdate = time.time()
        
        self.forcePartId = -1 # force on center of mass
        
        self.force = (0 ,0 ,0)
        
        self.linAcceleration = (0, 0, 0)
        self.linVelocity = (0, 0, 0)
        
        self.torque = (0, 0, 0)
        self.angMomentum = (0, 0, 0)
        self.angVelocity = (0,0,0)
        
        self.position = (0,5,0)
        self.orientation = Quaternion(0,0,0,1)
        self.orientation.normalize()
        #print "quaternion:", self.orientation.x, self.orientation.y, self.orientation.z, self.orientation.w
        
        #print "orientation: ", self.orientation.toAxisAngle()
        
        #print "rotation matrix: ", self.orientation.toRotationMatrix()
        
        self.loadModel(filename)
        self.calcCenterOfMass()
        self.moveToCenterOfMass()
        self.calcInertiaTensor()
        
    def calcCenterOfMass(self):
        massSum = 0
        positionSum = [0,0,0]
        
        for part in self.parts:
            massSum = massSum + part.mass
            
            positionSum[0] += (part.position[0] * part.mass ) #x
            positionSum[1] += (part.position[1] * part.mass ) #y
            positionSum[2] += (part.position[2] * part.mass ) #z
        
        positionSum[0] /= massSum
        positionSum[1] /= massSum
        positionSum[2] /= massSum
        
        self.mass = massSum 
        self.centerOfMass = tuple(positionSum)
        
        
        #print "mass: ",self.mass
        #print "center of mass:", self.centerOfMass
    
    def moveToCenterOfMass(self):
        for part in self.parts:
            part.position = ( part.position[0] - self.centerOfMass[0],
                              part.position[1] - self.centerOfMass[1],
                              part.position[2] - self.centerOfMass[2])
        self.centerOfMass = (0,0,0)
    
    def loadModel(self, filename):
        file = open( filename , 'r')
        stringList = file.readlines()
        
        for i in stringList:
            if i[0] != '#':
                chunkList = string.split(i)
                
                print chunkList
                
                position = ( float(chunkList[0]), float(chunkList[1]), float(chunkList[2]) )
                
                self.parts.append( ModelPart( position, float(chunkList[3])  ) )
                
        file.close()
        
    def calcInertiaTensor(self):
        result = zeros( (3, 3) , float_ )
        
        for part in self.parts:
            # J0 + mi*( ci' * ci * 1 - ci*ci'  )
            c = array( part.position ) #array from tuple
            q1 = cross( cross( c.transpose(), c ), identity(3) )
            q2 = cross( c, c.transpose() )
            
            J = part.calcInertiaTensor() + part.mass * ( q1 - q2 )
            
            result = result + J
        
        #return result
        self.inertiaTensor = result
    
    def draw(self, picking = False):
        
        glTranslatef( self.position[0], self.position[1], self.position[2] )
        
        #draw linear velocity
        glBegin(GL_LINES)
        glVertex3f( self.centerOfMass[0], self.centerOfMass[1], self.centerOfMass[2])
        glVertex3f( self.linVelocity[0] *5, self.linVelocity[1] *5, self.linVelocity[2] *5)
        glEnd()
                
        
        rotation = self.orientation.toAxisAngle()
        glRotatef( (rotation[3] / pi )* 180  , rotation[0], rotation[1], rotation[2] )

        #draw force
        glColor3f(1,0,0)
        force = dot( linalg.inv( array(self.orientation.toRotationMatrix() ) ), self.force  )
        
        if self.forcePartId>-1:
            glBegin(GL_LINES)
            #glVertex3f( self.centerOfMass[0], self.centerOfMass[1], self.centerOfMass[2])
            glVertex3f( self.parts[self.forcePartId].position[0], self.parts[self.forcePartId].position[1], self.parts[self.forcePartId].position[2])
            glVertex3f( self.parts[self.forcePartId].position[0]+force[0], self.parts[self.forcePartId].position[1]+force[1], self.parts[self.forcePartId].position[2]+force[2])
            glEnd()

        for p in range(0, len(self.parts) ):
            self.parts[p].draw( picking, p+1)
            #line to center of mass
            glBegin(GL_LINES)
            glVertex3f( self.centerOfMass[0], self.centerOfMass[1], self.centerOfMass[2])
            glVertex3f( self.parts[p].position[0], self.parts[p].position[1], self.parts[p].position[2])
            glEnd()
            
        self.calcPhysics()
            

    def calcPhysics(self):
        t = (time.time() - self.lastUpdate )  #time
        
        #calculate linear acceleration
        self.linAcceleration = ( self.force[0] / self.mass, self.force[1] / self.mass, self.force[2] / self.mass)
        
        #update linear velocity
        self.linVelocity = (self.linVelocity[0] + self.linAcceleration[0] * t,
                            self.linVelocity[1] + self.linAcceleration[1] * t,
                            self.linVelocity[2] + self.linAcceleration[2] * t)

        #update position
        self.position = (self.position[0] + self.linVelocity[0] * t,
                         self.position[1] + self.linVelocity[1] * t,
                         self.position[2] + self.linVelocity[2] * t)
        
        #self.force is in global coordinates
        force = dot( linalg.inv( array(self.orientation.toRotationMatrix() ) ), self.force  )
        
        #calculate torque
        if self.forcePartId!=-1 : #apply torque
            r = array( [self.parts[self.forcePartId].position[0] - self.centerOfMass[0],
                 self.parts[self.forcePartId].position[1] - self.centerOfMass[1],
                 self.parts[self.forcePartId].position[2] - self.centerOfMass[2] ] )
            
            self.torque = cross(r, array( force ) )
            
        else:
            self.torque = (0,0,0)
        
        #update angular momentum
        self.angMomentum = (self.angMomentum[0] + ( self.torque[0] * t ),
                            self.angMomentum[1] + ( self.torque[1] * t ),
                            self.angMomentum[2] + ( self.torque[2] * t ))
        
        #TODO: angular acceleration
        
        #compute angular velocity
        av = dot( linalg.inv( self.inertiaTensor ), self.angMomentum )
        self.angVelocity = (av[0], av[1], av[2])
        
        #update orientation
        p = dot( array(self.orientation.getQuaterinonMatrix()), av)
        p = dot( p, 0.5*t )
        self.orientation.add( p.tolist() )
        self.orientation.normalize()
        
        #update inertia tensor
        q = Quaternion( p[0], p[1], p[2], p[3])
        q.normalize()
        r = array( q.toRotationMatrix() )
        self.inertiaTensor = dot( r , dot( self.inertiaTensor, transpose(r) )  )
        
        #position restriction + friction
        if self.position[0] < -10 or self.position[0] > 10 or \
            self.position[1] < 0 or self.position[1] > 10 or \
            self.position[2] < -10 or self.position[2] > 10:
            
            self.linVelocity = ( - self.linVelocity[0] * 0.9,
                                 - self.linVelocity[1] * 0.9,
                                 - self.linVelocity[2] * 0.9)
            self.angMomentum = (self.angMomentum[0] * 0.9,
                            self.angMomentum[1] * 0.9,
                            self.angMomentum[2] * 0.9)
            #self.angMomentum = (0,0,0)
        
        #x
        if self.position[0] < -10:
            self.position = ( -10,
                              self.position[1],
                              self.position[2])
            
        if self.position[0] > 10:
            self.position = ( 10,
                              self.position[1],
                              self.position[2])
        #y
        if self.position[1] < 0:
            self.position = ( self.position[0],
                              0,
                              self.position[2])
            
        if self.position[1] > 10:
            self.position = ( self.position[0],
                              10,
                              self.position[2])
        #z
        if self.position[2] < -10:
            self.position = ( self.position[0],
                              self.position[1],
                              -10)
            
        if self.position[2] > 10:
            self.position = ( self.position[0],
                              self.position[1],
                              10)
        
        self.lastUpdate = time.time()
        
        # remove force if time is up
        if self.forceEndTime < self.lastUpdate:
            self.force = (0,0,0)
            self.forceEndTime = -1
            #print "force time up"
        
    def applyForce(self, (x,y,z), particleID = -1 ):
        
        self.force = (x,y,z)
        self.forcePartId = particleID
        
        #color
        for part in self.parts:
            part.color = (0,1,1)
        self.parts[self.forcePartId].color = (1,0,0)
        
            
            
            
            
            
            
            
            
        