'''
Created on 16.1.2011

@author: myron
'''
import math
from numpy import *
from OpenGL.GL import *
from OpenGL.GLU import *

class ModelPart(object):
    '''
    Defines one part (sphere) of model
    
    radius
    density
    position
    mass
    volume
    inertia tensor
    '''


    def __init__(self, position, radius, density=1.0):
        '''
        Constructor
        
        position - tuple (x,y,z)
        '''
        self.position = position
        self.radius = radius
        self.density = density
        
        self.volume = (4.0/3.0)* math.pi * pow(radius, 3)
        self.mass = self.volume * density
        
        self.color = (0 , 1 , 1)
        
        #print "mass:", self.mass
        #print "volume:", self.volume
        
        #print "inertia tensor: ", self.calcInertiaTensor()
        
        self.quadric = gluNewQuadric()

        
    def calcInertiaTensor(self):
        J = (2.0/5.0) * self.mass * pow(self.radius, 2)
        return array( [[J,0,0], [0,J,0], [0,0,J]] )
    
    def draw(self, picking = False, id = 0):
        
        glColor3fv( self.color )
        glTranslatef( self.position[0], self.position[1], self.position[2] )
        if picking:
            glLoadName( id )
        gluSphere( self.quadric, self.radius, 32, 32)
        glTranslatef( -self.position[0], -self.position[1], -self.position[2] )
        
        
        