'''
Created on 18.1.2011

@author: myron
'''

from math import *

class Quaternion(object):
    '''
    classdocs
    '''


    def __init__(self, x=0.0, y=0.0, z=0.0, w=1.0):
        '''
        Constructor
        '''
        self.x = x
        self.y = y
        self.z = z
        self.w = w
    
    def __str__(self):
        return "x: " + str(self.x) + " y: " + str(self.y) + " z: " + str(self.z) + " w: " + str(self.w) 
    
    def normalize(self):
        mag = self.x * self.x + self.y * self.y + self.z * self.z + self.w * self.w
        mag = sqrt(mag)
        
        if mag!=0:
            self.x /= mag
            self.y /= mag
            self.z /= mag
            self.w /= mag
        else:
            self.x = 0
            self.y = 0
            self.z = 0
            self.w = 1         
        
    def getConjugate(self):
        return Quaternion( -self.x, -self.y, -self.z, self.w )
    
    def multiply(self, q):
        '''
        q - quaternion to multiply
        '''
        x = self.w * q.x + self.x * q.w + self.y * q.z - self.z * q.y
        y = self.w * q.y + self.y * q.w + self.z * q.x - self.x * q.z
        z = self.w * q.z + self.z * q.w + self.x * q.y - self.y * q.x
        w = self.w * q.w - self.x * q.x - self.y * q.y - self.z * q.z
        
        return Quaternion(x, y, z ,w)
    
    def fromAxisAngle(self, (x,y,z), a ):
        '''
        (x,y,z) - tuple of vector
        a - angle
        '''
        a *= 0.5
        
        #normalize vector
        mag = x*x + y*y + z*z
        mag = sqrt(mag)
        x /= mag
        y /= mag
        z /= mag 
        
        sinA = sin(a)
        
        self.x = x * sinA
        self.y = y * sinA
        self.z = z * sinA
        self.w = cos(a)
        
    def toAxisAngle(self):
        '''
        returns (x,y,z,angle)
        '''

        mag = sqrt(self.x * self.x + self.y * self.y + self.z * self.z)
        
        if mag != 0: #maybe problem because of ptyhon float representation
            x = self.x / mag
            y = self.y / mag
            z = self.z / mag
        else:
            x = 0.0
            y = 0.0
            z = 0.0
        a = acos(self.w) * 2.0
        
        return (x, y, z, a)
        
    def toRotationMatrix(self):
        
        x2 = self.x * self.x
        y2 = self.y * self.y
        z2 = self.z * self.z
        xy = self.x * self.y
        xz = self.x * self.z
        yz = self.y * self.z
        wx = self.w * self.x
        wy = self.w * self.y
        wz = self.w * self.z
        
        return [ [1.0 - 2.0 * (y2 + z2), 2.0 * (xy - wz), 2.0 * (xz + wy)],
               [2.0 * (xy + wz), 1.0 - 2.0 * (x2 + z2), 2.0 * (yz - wx)],
               [2.0 * (xz - wy), 2.0 * (yz + wx), 1.0 - 2.0 * (x2 + y2)] ]

    def getQuaterinonMatrix(self):
        return [[ self.w , - self.z , self.y ],
                [ self.z , self.w , - self.x ],
                [ - self.y , self.x , self.w ],
                [ - self.x , - self.y , - self.z ]]
        
    def add(self, q):
        if( len(q)!= 4):
            raise Exception("bad parameter for quaternion.add method")
        
        self.x += q[0]
        self.y += q[1]
        self.z += q[2]
        self.w += q[3]
        